//---------------------------------------------------------------------------

#pragma hdrstop

#include "DSO183.h"
#include "..\miniscope\device\MiniscopeDevice.h"
#include "Signal.h"

#include <stdio.h>
#include <string.h>
#include <stdarg.h>

#include "Mutex.h"
#include "ScopedLock.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)

TestScope testScope;
Mutex mutex;
Signal signal[CHANNELS];


extern unsigned int iBuf[];
extern float fSens[];
extern float fSampl[];


TestScope::TestScope():
	workerThread(NULL),
	bManualTrigger(false),
	lastTriggerDistance(0)
{
	bDeviceOpened = false;
	bRunning = false;
	bStopRequest = false;
	mode = TRIGGER_MODE_AUTO;
	callbackData = NULL;
	callbackLog = NULL;
	callbackConnect = NULL;
	callbackTrigger = NULL;
	settings.samples = 512;
	for (int i=0; i<CHANNELS; i++)
	{
		settings.fDivY[i] = 0.001;
	}
	settings.fDivX = 0.00001;
	settings.triggerChannel = 0;
	settings.triggerType = TRIGGER_SLOPE_UP;


}

TestScope::~TestScope()
{
}

void TestScope::log(char *lpData, ...)
{
    static char buf[256]; //determines max message length
    va_list ap;
    va_start(ap, lpData);
    vsnprintf(buf, sizeof(buf), lpData, ap);
    va_end(ap);
    buf[sizeof(buf)-1] = 0;

    if (callbackLog)
        callbackLog(buf);
}

int TestScope::Connect(void)
{
	ConnectCb(DEVICE_CONNECTED, "Test signal plugin");
	workerThread = new WorkerThread(false);
	return 0;
}

int TestScope::Disconnect(void)
{
	if (workerThread)
	{
		workerThread->Terminate();
		Sleep(50);
		delete workerThread;
		workerThread = NULL;
	}
	ConnectCb(DEVICE_DISCONNECTED, "Disconnected");
	return 0;
}

int TestScope::Run(bool state)
{
	bRunning = state;
	return 0;
}

int TestScope::SetTriggerMode(enum E_TRIGGER_MODE m)
{
	mode = m;
	return 0;
}

int TestScope::ManualTrigger(void)
{
	bManualTrigger = true;
	return 0;
}

int TestScope::SetTriggerType(int channel, enum E_TRIGGER_TYPE type)
{
	settings.triggerChannel = channel;
	settings.triggerType = type;
	return 0;
}

int TestScope::EnableChannel(int channel, bool state)
{
	ScopedLock<Mutex> lock(mutex);
	settings.channelEnabled[channel] = state;
	return 0;
}

int TestScope::SetCouplingType(int channel, enum E_COUPLING_TYPE type)
{
	return 0;
}

int TestScope::SetBufferSize(int size)
{
	settings.samples = size;
	return 0;
}

int TestScope::SetSensitivity(int channel, int id)
{
	if (id < 0 || id > 5)
		return -1;
	settings.fDivY[channel] = fSens[id];
	return 0;
}

int TestScope::SetSampling(int id)
{
	if (id < 0 || id > 10)
		return -1;
	settings.fDivX = fSampl[id];
	return 0;
}

int TestScope::SetTriggerLevel(int level)
{
	if (level < -100 || level > 100)
		return -1;
	settings.triggerval = 255 - ((255 * (level+100))/200);
	return 0;
}

int TestScope::SetTriggerPosition(int pos)
{
	if (pos < 0 || pos > 100)
		return -1;
	settings.triggerpos = pos;
	return 0;
}

void TestScope::handle(void)
{
	if (bRunning || bManualTrigger)
	{
		ScopedLock<Mutex> lock(mutex);
		
		int enabledChannelsCnt = 0;
		for (int i=0; i<CHANNELS; i++)
		{
			if (settings.channelEnabled[i] == false)
				continue;
			short *tmp = signal[i].Generate(settings.samples,
				settings.fDivX, settings.fDivY[i]);
			memcpy(buffer + (enabledChannelsCnt * settings.samples), tmp, sizeof(buffer[0]) * settings.samples);
			enabledChannelsCnt++;
		}

		callbackTrigger();
		int size = enabledChannelsCnt * settings.samples * sizeof(short);
		callbackData(buffer, size);
    }
	bManualTrigger = false;	
}

int TestScope::TestTrigger(short* output_buffer, int len)
{
	int pos = -1;
	int level = (int)(2.55f * (float)settings.triggerval);	//-100...100 => -127...127 range
	if (bManualTrigger || mode == TRIGGER_MODE_CONTINUOUS)
	{
		pos = 0;
	}
	else
	{
		switch (settings.triggerType)
		{
		case TRIGGER_SLOPE_UP:
			for (int i=0; i<len-6; i++)
			{
				if (output_buffer[i] < level && output_buffer[i+1] <= level && output_buffer[i+2] <= level &&
					output_buffer[i+3] >= level && output_buffer[i+4] >= level && output_buffer[i+5] > level)
				{
					//log("i = %d, len = %d, %d %d %d %d %d %d", i, len, output_buffer[i], output_buffer[i+1], output_buffer[i+2],
					//	output_buffer[i+3], output_buffer[i+4], output_buffer[i+5]);
					pos = i;
					break;
				}
			}
			break;
		case TRIGGER_SLOPE_DOWN:
			for (int i=0; i<len-6; i++)
			{
				if (output_buffer[i] > level && output_buffer[i+1] >= level && output_buffer[i+2] >= level &&
					output_buffer[i+3] <= level && output_buffer[i+4] <= level && output_buffer[i+5] < level)
				{
					//log("i = %d, len = %d, %d %d %d %d %d %d", i, len, output_buffer[i], output_buffer[i+1], output_buffer[i+2],
					//	output_buffer[i+3], output_buffer[i+4], output_buffer[i+5]);
					pos = i;
					break;
				}
			}
			break;
		default:
			break;
		}
	}
	if (mode == TRIGGER_MODE_AUTO)
	{
		if (pos == -1)
		{
			lastTriggerDistance += len;
			// generate auto-trigger when there is no trigger for some time
			if (lastTriggerDistance >= 15 * settings.samples)
			{
				pos = 0;
				lastTriggerDistance = 0;
			}
		}
		else
		{
			lastTriggerDistance = 0;
		}
	}
	return pos;
}


